@@ -13,15 +13,15 @@ namespace FrictionQPotSpringBlock {
 
 namespace detail {
 
-    inline std::string unquote(const std::string& arg)
-    {
-        std::string ret = arg;
-        ret.erase(std::remove(ret.begin(), ret.end(), '\"'), ret.end());
-        return ret;
-    }
-
+inline std::string unquote(const std::string& arg)
+{
+    std::string ret = arg;
+    ret.erase(std::remove(ret.begin(), ret.end(), '\"'), ret.end());
+    return ret;
 }
 
+} // namespace detail
+
 inline std::string version()
 {
     return detail::unquote(std::string(QUOTE(FRICTIONQPOTSPRINGBLOCK_VERSION)));
